package org.openscm.kundo.plugins

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.openscm.kundo.common.PropertyUtils
import org.openscm.kundo.common.recipe.*
import org.apache.tools.ant.BuildException

 /**
 * Init Delegate
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to load properties from properties files in a hiearchical fashion.
 * 
 * <p>The init delegate reads a default properties file and from that optionally loads an alternative hierarchy of
 * properties files specified in said default build.properties file.</p>
 * 
 */
/* 
 * @ant-target doInit
 */
class InitDelegate extends AbstractPluginTargetDelegate {

	/*
	 * The location of the default build.properties file to load
	 */	
	private String defaultPropertiesFile
	
	private fs = File.separator
	
	/*
	 * A set of properties file locations grouped by key and index. Format: [plugin].[key].[index].[fieldname]
	 * @indexed-ant-property indexed
	 */	
	Map indexedProps	
	
	/**
	 * Constructor
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	InitDelegate( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */
	void doCheck(){
		assert defaultPropertiesFile != null : "An init.default.properties.file must be specified"
	}
	
	/**
	 * Setter method for member variable defaultPropertiesFile
	 * @param defaultPropertiesFile	String representation the project's default build.properties file
	 */	
	void setDefaultPropertiesFile( String defaultPropertiesFile ){
		this.defaultPropertiesFile = defaultPropertiesFile 
	}	
	
	/**
	 * delegate implementation method to be called by doExecute
	 */
	void doInit(){
		if( log.isDebugEnabled() ){
			log.debug( "Entering -InitDelegate.doInit( )" )
		}
		
		def fullFile = "${ant.project.getProperty( 'basedir' )}${fs}${defaultPropertiesFile}"
		def fileTest = new File( fullFile )
		if( !fileTest.exists() ){
			throw new BuildException( "build.properties file ${fullFile} does not exist" )
		}
		ant.property( file:fullFile )
		
		//def indexedProps = PropertyUtils.getPrefixedIndexedProperties( ant, "init", new Properties( ant.project.properties ) )
		indexedProps = getIndexedProps()		
		
		/*def propertyFileList = indexedProps.get( "properties" )
		propertyFileList.each{ dynamicPropertyObject ->
			println "${dynamicPropertyObject.location}"
			ant.property( file:dynamicPropertyObject.location )
		}*/

		indexedProps.each{ key, value ->
			ant.property( file:value )
		}
		
		//check if a helper strategy is defined
		//if so we already handle all that
		//if not we will be using the recipe for metadata
		def helperStrategy = ant.project.getProperty( "project.helper.strategy" )
		
		if( helperStrategy == null ){
			
			helperStrategy = ant.project.getProperty( "global.helper.strategy" )
			if( helperStrategy == null ){
				this.manageArtifactVersionProperty()
			}
		}

		if( log.isDebugEnabled() ){
			log.debug( "Exiting -InitDelegate.doInit( )" )
		}
	}
	
	/**
	 * This method was added to ensure that the indexed properties were loaded by order of the key
	 * It appears that the PropertyUtils method may not return the list of indexed properties in
	 * index order. This may not be a problem for the majority of cases but in this case it is.
	 * The PropertyUtils method will need looking at to see what can be done about this.
	 * @return Map a map of indexed properties sorted by key and therefore by index
	 */
	Map getIndexedProps(){
		int i = 0 
		def returnValue = new TreeMap(  )
		while( ant.project.getProperty( "init.properties.${i}.location" ) != null ){
			def value = ant.project.getProperty( "init.properties.${i}.location" )
			returnValue.put( "init.properties.${i}.location", value )
			i++
		}
		
		return returnValue
	}
	
	/**
	 * This method will try and load the artifact.development.property
	 * or artifact.release.property to a global variable, if it doesn't exist
	 * it will search for a parent project's declaration, if that doesn't exist
	 * an exception will be thrown
	 */
	private void manageArtifactVersionProperty(){
		
		//now we need to handle the externalised artifact version property
		//see if a release property has been entered on the command line
		def artifactReleaseVersion = ant.project.getProperty( "artifact.release.version" )
		
		//if there is no release version we must be using the development version
		if( artifactReleaseVersion == null ){
			
			//check if the development version has been loaded from this project's build.properties file
			def artifactDevelopmentVersion = ant.project.getProperty( "artifact.development.version" )	
			if( artifactDevelopmentVersion != null ){
				
				//if the version was in this project's build.properties then add it as a global property for all other plug-ins
				ant.property( name : "global.artifact.development.version", value : artifactDevelopmentVersion )
			}else{
				
				//if we don't have a development version in this project's build.properties then try and find it in the parent
				this.readVersionPropertyFromParent()
			}
		}else{
			
			//if there is a release version specified then add it as a global variable for use by all other plug-ins
			ant.property( name : "global.artifact.release.version", value : artifactReleaseVersion )
		}		
	}
	
	/**
	 * This method will try and retrieve the recipe hierarchy from the ant project
	 * If no parent section exists in the recipe then an exception is thrown
	 * If a parent section does exist, this method will be called recursively until
	 * the artifact.development.version property is found in a build.properties file
	 */
	private void readVersionPropertyFromParent(){
		
		def recipeHierarchy = buildContext.get( "recipe.hierarchy" )
		
		def parent = recipeHierarchy.getParent()
		
		if( parent == null ){			
			throw new MissingPropertyException( "An artifact.development.version property needs to be specified in this project's or a parent project's build.properties file." )
		}
		
		def basedir = ant.project.getProperty( "basedir" )
		
		def parentRecipe = new File ( "${basedir}${File.separator}${parent.getLocation()}" )
		def buildPropertiesFile = new File( "${parentRecipe.getParentFile().getAbsolutePath()}${File.separator}build.properties" )
		
		def props = PropertyUtils.getPropertiesFromInputStream( new FileInputStream( buildPropertiesFile ) )
		
		def artifactDevelopmentVersion = props.getProperty( "artifact.development.version" )
		
		if( artifactDevelopmentVersion == null ){
			this.readVersionPropertyFromParent()
		}
		
		ant.property( name : "global.artifact.development.version", value : artifactDevelopmentVersion )
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting -InitDelegate.readVersionPropertyFromParent( )" )
		}		
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doInit()
	}
	
}
